#include <iostream>
#include <sndfile.h>
#include <cstdlib>
using namespace std;

bool readBool(string name);
int readValue(string name);

/**
 * This program converts the Audition text file format to a wave file. This
 * reads the text file in from stdin so that the output of the DFT program can
 * be piped through this to produce a wave file directly without any
 * intermediate text file.
 *
 * @author Robert Flack
 */

int main(int argc, char** argv)
{
	if (argc!=2)
	{
		cerr << "Please run this program: "<<argv[0]<<" [wavefile]" << endl;
		return -1;
	}

	SF_INFO sfinfo;
	int bitspersample;
	int frames;
	sfinfo.frames=frames=readValue("SAMPLES");
	bitspersample=readValue("BITSPERSAMPLE");
	sfinfo.channels=readValue("CHANNELS");
	sfinfo.samplerate=readValue("SAMPLERATE");
	readBool("NORMALIZED");

	int bitformat=SF_FORMAT_PCM_16;
	if (bitspersample==8)
		bitformat=SF_FORMAT_PCM_S8;
	else if (bitspersample==16)
		bitformat=SF_FORMAT_PCM_16;
	else if (bitspersample==24)
		bitformat=SF_FORMAT_PCM_24;
	else if (bitspersample==32)
		bitformat=SF_FORMAT_PCM_32;

	sfinfo.format=SF_FORMAT_WAV | bitformat;

	SNDFILE* sf=sf_open(argv[1], SFM_WRITE, &sfinfo);
	if (!sf)
	{
		cerr << "Error creating sound file "<<argv[1]<<endl;
		return -2;
	}

	short *sdata = new short[sfinfo.channels];
	for (int i=0;i<frames;i++)
	{
		for (int j=0;j<sfinfo.channels;j++)
		{
			cin >> sdata[j];
		}
		if (!cin.good())
		{
			cerr << "Early end of input - text file broken?" << endl;
			break;
		}
		sf_write_short(sf, sdata, sfinfo.channels);
	}
	delete [] sdata;
	sf_close(sf);
	return 0;
}

bool readBool(string name)
{
	string tmp;
	cin >> tmp;
	if ((name+":")!=tmp)
	{
		cerr << "Error, expected \""<<name<<"\""<<endl;
		exit(-2);
	}
	cin >> tmp;
	return (tmp=="TRUE");
}

int readValue(string name)
{
	string tmp;
	int val;
	cin >> tmp;
	if (tmp!=(name+":"))
	{
		cerr << "Error, expected "<<name<<endl;
		exit(-2);
	}
	cin >> val;
	return val;
}

