#include <iostream>
#include <sndfile.h>
using namespace std;

/**
 * This utility converts a wave file to the Audition txt file format. This will
 * allow me to pipe the output of this program to the main DFT program for the
 * first part of the assignment.
 *
 * @author Robert Flack
 */

int main(int argc, char** argv)
{
	if (argc!=2)
	{
		cerr << "Please run this program: "<<argv[0]<<" [wavefile]" << endl;
		return -1;
	}

	SF_INFO sfinfo;
	SNDFILE* sf=sf_open(argv[1], SFM_READ, &sfinfo);
	if (!sf)
	{
		cerr << "Error opening sound file "<<argv[1]<<endl;
		return -2;
	}

	cout << "SAMPLES:\t" << sfinfo.frames << endl;
	int bitspersample=16; // Default to 16
	if (sfinfo.format & SF_FORMAT_PCM_S8)
		bitspersample=8;
	else if (sfinfo.format & SF_FORMAT_PCM_16)
		bitspersample=16;
	else if (sfinfo.format & SF_FORMAT_PCM_24)
		bitspersample=24;
	else if (sfinfo.format & SF_FORMAT_PCM_32)
		bitspersample=32;
	
	cout << "BITSPERSAMPLE:\t" << bitspersample << endl;
	cout << "CHANNELS:\t" << sfinfo.channels << endl;
	cout << "SAMPLERATE:\t" << sfinfo.samplerate << endl;
	cout << "NORMALIZED:\tFALSE"<<endl;
	
	short *sdata = new short[sfinfo.channels];
	for (int i=0;i<sfinfo.frames;i++)
	{
		if (sf_read_short(sf, sdata, sfinfo.channels)!=sfinfo.channels)
		{
			cerr << "Early end of sound file encountered" << endl;
			break;
		}
		for (int j=0;j<sfinfo.channels;j++)
		{
			if (j>0)
				cout << "\t";
			cout << sdata[j];
		}
		cout << endl;
	}
	delete [] sdata;
	sf_close(sf);
	return 0;
}
